// video.cpp

//    Program Name: Scan3d    Plugin for 3d video scanning using Mach3.
//    Copyright (C) 2008,2009  Thomas Hubin

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License gpl.txt for more details.

//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

//    Author: Tom Hubin, thubin@earthlink.net
//    13151 Larchdale Rd, #5
//    Laurel, MD 20708-1719

#include "stdafx.h"
#include "resource.h"
#include "PluginTestDlg.h"
#include "CustomPlugin.h"


#include "Utility.h"
#include "dbg.h"

#include <vfw.h>
#include <math.h>

    // local headers

#include "video.h"

    // local definitions

#define CALIBRATION 1
#define PART 2

    // local constants

const double degPerRad = 90.0/asin(1.0) ;

    // useful colors for graphics

static const COLORREF rgbBlack = RGB(0,0,0) ;
static const COLORREF rgbWhite = RGB(255,255,255) ;
static const COLORREF rgbRed = RGB(255,0,0) ;
static const COLORREF rgbGreen = RGB(0,255,0) ;
static const COLORREF rgbBlue = RGB(0,0,255) ;
static const COLORREF rgbMagenta = RGB(255,0,255) ;
static const COLORREF rgbGray = RGB(127,127,127) ;

    // local variables

static HWND hwndScan3dDlgParent = 0 ;
static CWnd cwndScan3dDlgParent  ;
	  
static CPluginTestDlg *pScan3dDlg = NULL ;  //  pointer to dialog box
static HWND hScan3dDlg = 0 ;				 // handle for dialog box

static HWND hwndVideoRect1 = 0 ;

static BOOL fIsScan3dOpen = FALSE ;

    // description of cameras for user selection

struct CAMERADESCRIPTION
{
    char szName[81] ;
	char szVersion[41] ; 
} ; 

CAMERADESCRIPTION cameraDescription[10] ;

// bitmap data

static BITMAPINFOHEADER bmiImage1 ;
static unsigned char *pImage1 = NULL ;  // allocated memory must be freed when done

struct SAMPLE
{
    double droX, droY, droZ ;  //  dro coordinates when sample taken
	unsigned int h, v  ;  //  image coordinates of sample
	unsigned char red, green, blue ;  //  sample value
	unsigned int gray ;  //  equivalent gray shade
} ;

static SAMPLE *pSamples = NULL ;  //  pointer to array of samples
static int maxSamples = 0 ;  //  max number of samples array can take
static int nSamples = 0 ;  //  number of valid samples

    // calibration data

static const int nCoef = 6 ;	// number of coefficients
static double A[nCoef] ; // for calculating x
static double B[nCoef] ; // for calculating y
static double C[nCoef] ; // for calculating z

    // graphic windows

static int pictureTop = 40 ;
static int picture1Left = 10 ;
static int picture2Left = 340 ;
static int picture3Left = 670 ;
//static int thresholdScrollLeft = 1000 ;
static int picture4Left = 1000 ;
 
    // local prototypes

static int FetchImage ( void ) ;

static int OrientImage ( void ) ;

static int ShowImage ( void ) ;

static int AnalyzeImage ( void ) ;

static int GraphShape ( void ) ;

static int GraphIntensities ( void ) ;

static int DoMotion ( int scanType, FILE *outFile ) ;

//---------------------------------------

void ShowOpenSourceNotice ( void )
{
	char OpenSourceMsg[1025] ;

	strcpy ( OpenSourceMsg, "\nProgram Name: Scan3d    Plugin for 3d video scanning using Mach3." ) ;
    strcat ( OpenSourceMsg, "\nCopyright (C) 2008,2009  Thomas Hubin" ) ;

	strcat ( OpenSourceMsg, "\n" ) ;
    strcat ( OpenSourceMsg, "\nThis program is free software: you can redistribute it and/or modify" ) ;
    strcat ( OpenSourceMsg, "\nit under the terms of the GNU General Public License as published by" ) ;
    strcat ( OpenSourceMsg, "\nthe Free Software Foundation, either version 3 of the License, or" ) ;
    strcat ( OpenSourceMsg, "\n(at your option) any later version." ) ;
 
	strcat ( OpenSourceMsg, "\n" ) ;    
	strcat ( OpenSourceMsg, "\nThis program is distributed in the hope that it will be useful," ) ;    
    strcat ( OpenSourceMsg, "\nbut WITHOUT ANY WARRANTY; without even the implied warranty of" ) ;
    strcat ( OpenSourceMsg, "\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the" ) ;
    strcat ( OpenSourceMsg, "\nGNU General Public License gpl.txt for more details." ) ;

	strcat ( OpenSourceMsg, "\n" ) ;
    strcat ( OpenSourceMsg, "\nYou should have received a copy of the GNU General Public License" ) ;
    strcat ( OpenSourceMsg, "\nalong with this program.  If not, see <http://www.gnu.org/licenses/>." ) ;

	strcat ( OpenSourceMsg, "\n" ) ;
	strcat ( OpenSourceMsg, "\nAuthor: Tom Hubin                Email: thubin@earthlink.net" ) ;
	strcat ( OpenSourceMsg, "\n13151 Larchdale Rd, #5" ) ;
	strcat ( OpenSourceMsg, "\nLaurel, MD 20708-1719" ) ;

	MessageBox ( NULL, OpenSourceMsg, "Open Source Notice", MB_OK ) ;
}

void CALLBACK EXPORT TimerProc
(
   HWND hWnd,      // handle of CWnd that called SetTimer
   UINT nMsg,      // WM_TIMER
   UINT nIDEvent,  // timer identification
   DWORD dwTime    // system time
)
{
	if ( ( hWnd == hScan3dDlg ) && ( nMsg == WM_TIMER ) && ( nIDEvent == IDTIMER ) )
	{
        UpdateScan3d() ;   
	}
}

int StartTimer ( void )
{    
	int rtc = 0 ;

	SetTimer ( hScan3dDlg, IDTIMER, 100, TimerProc ) ;

    return(rtc) ; 
}

int StopTimer ( void ) 
{
    int rtc = 0 ;

    KillTimer ( hScan3dDlg, IDTIMER ) ;

	return(rtc) ;
}

int OpenScan3d ( void )
{
	{
	    static bool OpenSourceNoticeShown = FALSE ; 
		if ( !OpenSourceNoticeShown )
		{
            ShowOpenSourceNotice() ;
			OpenSourceNoticeShown = TRUE ;
		}
	}

    int rtc = 0 ; // error return code
	char tmpString[255] ;

	if ( pScan3dDlg != NULL )
	{
	    pScan3dDlg->ShowWindow(SW_SHOW) ;
		rtc = 1 ;
		goto quit ;
	}

	    // create video dialog under Mach3

//	hwndScan3dDlgParent = GetMach3MainWindow(); // get Mach3 main window handle
	hwndScan3dDlgParent = GetDesktopWindow(); // get Desktop Window to use as dialog parent
	if (hwndScan3dDlgParent == 0)
	{
		MessageBox ( NULL, "Cannot find Mach handle", NULL, MB_OK ) ;
		return(1) ;
	}	  
    DbgMsg(("Found Mach3 main window"));

	if ( !cwndScan3dDlgParent.Attach(hwndScan3dDlgParent) )
	{
		MessageBox ( NULL, "Unable to attach", NULL, MB_OK ) ;
		return(3) ;
	}
    DbgMsg(("Attach ok"));

	pScan3dDlg = new CPluginTestDlg ; // allocate heap memory for dialog
	if ( pScan3dDlg == NULL )
	{
		MessageBox ( NULL, "Unable to allocate heap memeory for pScan3dDlg", NULL, MB_OK ) ;
		cwndScan3dDlgParent.Detach() ;
        return(4) ; 
	}	
    DbgMsg(("Heap memory allocated for pScan3dDlg"));

		// create scan3d dialog

	BOOL fTmp = pScan3dDlg->Create(IDD_SCAN3D, &cwndScan3dDlgParent) ;
	if ( !fTmp ) 
    {
        DbgMsg(("Failed to create Scan3d dialog box"));
 
		MessageBox ( NULL, "Unable to create Scan3d Dialog Box", tmpString, MB_OK ) ;
		delete pScan3dDlg ; // free memory from heap
        pScan3dDlg = NULL ;
		cwndScan3dDlgParent.Detach() ;
		return(5) ;
	}
    DbgMsg(("Created Scan3d dialog box"));

	pScan3dDlg->ShowWindow(SW_SHOW) ;	// show the dialog box
	LoadScan3dData ( "Scan3d.ini" ) ;   // Fetch data from ini file
	pScan3dDlg->UpdateData(FALSE) ;     // freshen data in dialog box 

	hScan3dDlg = pScan3dDlg->GetSafeHwnd() ;  // dialog box is parent of video window
	if ( !hScan3dDlg )
	{
        DbgMsg(("Unable to make hScan3dDlg")) ; 
		MessageBox ( NULL, "Unable to make hScan3dDlg", NULL, MB_OK ) ;
		rtc = 4 ;
		goto quit ;
	}
	DbgMsg(("created hScan3dDlg"));

	CreateVideoWindow() ;

quit:
	if ( rtc == 0 )
		fIsScan3dOpen = TRUE ;  // flag as successfully opened 

	return (rtc) ;
}

int CreateVideoWindow ( VOID )
{    
	int rtc = 0 ;
	char tmpString[254] ; // for temporary messages or text

			// resize other video windows

	MoveWindow ( GetDlgItem(hScan3dDlg,IDC_PICTURE2), picture2Left, pictureTop, 320, 240, TRUE ) ;
    MoveWindow ( GetDlgItem(hScan3dDlg,IDC_PICTURE3), picture3Left, pictureTop, 320, 240, TRUE ) ;
//	MoveWindow ( GetDlgItem(hScan3dDlg,IDC_THRESHOLD3), thresholdScrollLeft, pictureTop, 20, 256, TRUE ) ;
	MoveWindow ( GetDlgItem(hScan3dDlg,IDC_PICTURE4), picture4Left, pictureTop, 320, 240, TRUE ) ;

	if ( hwndVideoRect1 == 0 )
	{
		DWORD dwStyle = WS_CHILD | WS_VISIBLE ;  // not visible - just for acquiring the image
	    hwndVideoRect1 = capCreateCaptureWindow ( "Video Capture Window", dwStyle, picture1Left, pictureTop, 320, 240, hScan3dDlg, IDC_VIDEO1  ) ; 
		if ( hwndVideoRect1 == 0 )
	    {
	        DbgMsg(("Unable to create video rectangle")) ;
            MessageBox ( NULL, "Unable to create video rectangle", NULL, MB_OK ) ;
            rtc = 4 ;	
			goto quit ;
	    }
    	DbgMsg(("Video window created")) ;

		    // identify all available cameras

		int iCam ;
		BOOL fDriverConnected ;
//		for ( iCam = 0 ; iCam < 10 ; iCam++ )
//		{
//            fDriverConnected = capDriverConnect ( hwndVideoRect1, iCam ) ;
//            if ( fDriverConnected )
//			{
//                capDriverGetName ( hwndVideoRect1, cameraDescription[iCam].szName, sizeof(cameraDescription[iCam].szName) ) ;
//			    capDriverGetVersion ( hwndVideoRect1, cameraDescription[iCam].szVersion, sizeof(cameraDescription[iCam].szVersion) ) ;
//                capDriverDisconnect ( hwndVideoRect1, iCam ) ;
//			}
//			else
//			{
//                strcpy ( cameraDescription[iCam].szName, "" ) ;
//				strcpy ( cameraDescription[iCam].szVersion, "" ) ;
//			}
//		}

		    // connect to first available camera

		for ( iCam=0 ; iCam<10 ; iCam++ )
		{
            fDriverConnected = capDriverConnect ( hwndVideoRect1, iCam ) ;
            if ( fDriverConnected )
			{
			    char tmp[81] ;
	            sprintf ( tmp, "Video driver %i connected", iCam ) ;
	            DbgMsg((tmp)) ;
				break ;
			}
		}
		if ( !fDriverConnected )
	    {
		    DbgMsg(("Unable to connect video driver")) ;
			MessageBox ( NULL, "CreateVideoWindow()\n\nUnable to connect video driver", NULL, MB_OK ) ;
            DestroyWindow (hwndVideoRect1) ;
			hwndVideoRect1 = 0 ;
			rtc = 5 ;
		    goto quit;
	    }
	}

	    // camera connected

	capPreviewScale ( hwndVideoRect1, TRUE ) ;  // scale to fit window

	StartTimer() ;  // timer used to trigger video update 

quit:
    return(rtc) ;
}

int SelectCamera ( void )
{
	MessageBox ( NULL, "SelectCamera() not yet written.\n\nTry using Source to select a camera.", "Note:", MB_OK ) ;

    int cameraSelected = -1 ;    // init for no camera found
goto quit ;

    char szCameraList[(81+41)*10] ;

	sprintf ( szCameraList, "Camera List:\n" ) ;

	for ( int wIndex = 0; wIndex < 10; wIndex++ ) 
    {
//        if (capGetDriverDescription (wIndex, 
//			szDeviceName, sizeof(szDeviceName),
//            szDeviceVersion, sizeof(szDeviceVersion) ) ) 
        {
                // Append name to list of installed capture drivers
                // and then let the user select a driver to use.
            char tmp[255] ;
			sprintf ( tmp, "\n %i    %s    ver %s \n", wIndex, cameraDescription[wIndex].szName, cameraDescription[wIndex].szVersion ) ;
			strcat ( szCameraList, tmp ) ;
		}
    } 

    MessageBox ( NULL, szCameraList, "SelectCamera()", MB_OK ) ;

quit:
	return (cameraSelected) ;
}

//---------------------------------------

int FetchImage ( void )
{
    int rtc = 0 ;
	BOOL fClipboardOpen = FALSE ;
	HANDLE hDIB = 0 ;
	BITMAPINFOHEADER* pInfo = NULL ;

	if ( !capEditCopy ( hwndVideoRect1 ) )
	{
		static BOOL msgPosted = FALSE ;
		if ( !msgPosted )
		{   
            MessageBox ( NULL, "Unable to copy image to clipboard", NULL, MB_OK ) ;
			msgPosted = TRUE ;
		}
		rtc = 1 ;
		goto quit ;
	}

	fClipboardOpen = OpenClipboard(NULL) ;
	if ( !fClipboardOpen )
	{
		static BOOL msgPosted = FALSE ;
		if ( !msgPosted )
		{   
            MessageBox ( NULL, "Unable to open clipboard", NULL, MB_OK ) ;
			msgPosted = TRUE ;
		}
		rtc = 2 ;
		goto quit ;
	}

	hDIB = GetClipboardData (CF_DIB) ; 
	if ( hDIB == 0 )
	{
		static BOOL msgPosted = FALSE ;
		if ( !msgPosted )
		{   
            MessageBox ( NULL, "Unable to get clipboard handle", NULL, MB_OK ) ;
			msgPosted = TRUE ;
		}
		rtc = 3 ;
		goto quit ;
	}

	pInfo = (BITMAPINFOHEADER*) GlobalLock(hDIB) ; 
	if ( pInfo == NULL )
	{
		static BOOL msgPosted = FALSE ;
		if ( !msgPosted )
		{   
            MessageBox ( NULL, "Unable to get pointer to clipboard bitmap", NULL, MB_OK ) ;
			msgPosted = TRUE ;
		}
		rtc = 4 ;
		goto quit ;
	}

//	{
//		static BOOL msgPosted = FALSE ;
//	    if ( !msgPosted )
//	    { 
//	        char tmp[255] ;
//	        sprintf ( tmp, "biSize=%i    biWidth=%i    biHeight=%i    biBitCount=%i    biSizeImage=%li    biClrUsed=%li",
//				(int)pInfo->biSize, (int)pInfo->biWidth, (int)pInfo->biHeight, (int)pInfo->biBitCount , (long)pInfo->biSizeImage, (long)pInfo->biClrUsed ) ;
//            MessageBox ( NULL, tmp, "bitmap info", MB_OK ) ;
//			msgPosted = TRUE ;
//	    }
//	}

		// allocate memory for return of image copy

 	pImage1 = (unsigned char *) realloc ( pImage1, pInfo->biSizeImage ) ;
	if ( pImage1 == NULL )
	{
         MessageBox ( NULL, "Unable to allocate memory for image", NULL, MB_OK ) ;
		 rtc = 5 ;
		 goto quit ;
	}

	    // calculate address of clipboard image 

	int colorTableSize = 0 ; 
	if ( pInfo->biBitCount == 8 )
		colorTableSize = 4*pInfo->biClrUsed ;

    unsigned char* pSrc = (unsigned char*)pInfo + pInfo->biSize + colorTableSize ;
	
	memcpy ( pImage1, pSrc, pInfo->biSizeImage ) ;  // copy bitmap from clipboard

	memcpy ( &bmiImage1, pInfo, sizeof(bmiImage1) ) ;  // copy bitmap header

quit:
    if ( pInfo != NULL )
	{
        GlobalUnlock ( hDIB ) ; 
        pInfo = NULL ;
		hDIB = 0 ;
 	}

	if ( fClipboardOpen )
	{
        CloseClipboard() ;
		fClipboardOpen = FALSE ;
	}
	return(rtc);
}

int OrientImage ( void )
{
    int rtc = 0 ;
	static int previousRotation = -1 ;
    unsigned char *srcImage = NULL, *dstImage = NULL ; // pointers to source and destination images
	long srcWidth = 0, srcHeight = 0 ;  //  dimensions of source image
    long dstWidth = 0, dstHeight = 0 ;  //  dimensions of destination image

	int rotation = pScan3dDlg->videoRotation ;
	int mirror = pScan3dDlg->videoMirror ;

	int orientation = rotation ;
	if ( mirror )
		orientation += 4 ;
		
	if ( rotation != previousRotation )
	{
        if ( orientation & 1 )  //  odd multiple of 90 degrees
		{
			MoveWindow ( GetDlgItem(hScan3dDlg,IDC_PICTURE2), picture2Left, pictureTop, 240, 320, TRUE ) ;
 		}
		else
		{
			MoveWindow ( GetDlgItem(hScan3dDlg,IDC_PICTURE2), picture2Left, pictureTop, 320, 240, TRUE ) ;
		}
	}

	if ( orientation == 0 )  // orientation good as it is
	{
        rtc = 0 ;
		goto quit ;
	}

	    // allocate temporary memory for the new orientation

	dstImage = (unsigned char*) malloc( bmiImage1.biSizeImage ) ;
	if ( dstImage == NULL )
	{
        MessageBox ( NULL, "Unable to allocate memory in OrientImage()", NULL, MB_OK ) ;
		rtc = 1 ;
		goto quit ;
	}

    srcImage = pImage1 ;  //  point to the original image

	srcWidth = bmiImage1.biWidth ;
    srcHeight = bmiImage1.biHeight ;
    WORD nBytesPerPixel = bmiImage1.biBitCount/8 ;  //  number of bytes per pixel

	if ( rotation & 1 )  //  rotate odd multiple of 90 
	{
        dstWidth = srcHeight ;
		dstHeight = srcWidth ;
	}
	else                // rotate even multiple of 90 degrees
	{
        dstWidth = srcWidth ;
		dstHeight = srcHeight ;
	}

	for ( long dstV=0 ; dstV<dstHeight ; dstV++ )
	{
        for ( long dstH=0 ; dstH<dstWidth ; dstH++ )
		{
			long srcH, srcV ;  //  coordinates in source image
            switch (orientation)
			{
				case 0:  // no rotate and no mirror - copy as is
					srcH = dstH ;
                    srcV = dstV ;
					break ;

                case 1:  // rotate 90 degrees CW
					srcH = dstHeight - 1 - dstV ;
                    srcV = dstH ;
					break ;

			    case 2:  //  rotate 180 degrees
					srcH = dstWidth - 1 - dstH ;
                    srcV = dstHeight - 1 - dstV ; 
				    break ;

                case 3:  // rotate 90 degrees CCW
					srcH = dstV ;
                    srcV = dstWidth - 1 - dstH ;
					break ;

			    case 4:  //  no rotation - just horizontal mirror image
					srcH = dstWidth - 1 - dstH ;
                    srcV = dstV ; 
				    break ;

	            case 5:  // rotate 90 degrees CW then horizontal mirror
					srcH = dstHeight - 1 - dstV ;
                    srcV = dstWidth - 1 - dstH ;
					break ;

				case 6:  //  rotate 180 degrees then horizontal mirror
					srcH = dstH ;
                    srcV = dstHeight - 1 - dstV ; 
				    break ;

                case 7:  // rotate 90 degrees CCW then horizontal mirror
					srcH = dstV ;
                    srcV = dstH ;
					break ;

				default:    // should never be here
					rtc = 9 ;
					goto quit ;
			} 

			    // calculate pixel address for destination

			unsigned char *dstPtr = dstImage + ( dstV*dstWidth + dstH )*nBytesPerPixel ;

			    // calculate pixel address for source 

			unsigned char *srcPtr = srcImage + ( srcV*srcWidth + srcH )*nBytesPerPixel ;

                // copy 1 gray or 3 byte RGB pixel from source to destination

			for ( WORD iByte = 0 ; iByte < nBytesPerPixel ; iByte++ )
			{
			    *dstPtr++ = *srcPtr++ ;
			}
		}
	}

	    // transfer complete - do cleanup

	free ( pImage1 ) ;  //  delete old image
    pImage1 = dstImage ;  //  replace old image with new image
	dstImage = NULL ;
	bmiImage1.biWidth = dstWidth ;  //  update bitmapinfo header
	bmiImage1.biHeight = dstHeight ;

quit:
	if ( dstImage )
	{
        free ( dstImage ) ;
		dstImage = NULL ;
	}
	return(rtc) ;
}

int ShowImage ( void )
{
    int rtc = 0 ;
	HDC hdcPicture2 = 0 ;
	const int UNKNOWN = 0 ;
	const int PORTRAIT = 1 ;
	const int LANDSCAPE = 2 ;
	static int lastOrientation = UNKNOWN ;
	long dstWidth=0, dstHeight=0 ;  // dimensions of picture area

	HWND hwndPicture2 = GetDlgItem ( hScan3dDlg, IDC_PICTURE2 ) ;
    if ( hwndPicture2 == 0 )
    {
        MessageBox ( NULL, "No hwndPicture2", NULL, MB_OK ) ;
        rtc = 6 ;
		goto quit ;
    }

		// determine best output format and change if necessary

	LONG srcWidth = bmiImage1.biWidth ;
	LONG srcHeight = bmiImage1.biHeight ;
	if ( srcWidth > srcHeight )
	{
		dstWidth = 320 ;
		dstHeight = 240 ;
		if ( lastOrientation != LANDSCAPE )
			MoveWindow ( hwndPicture2, picture2Left, pictureTop, dstWidth, dstHeight, TRUE ) ;
		lastOrientation = LANDSCAPE ;
	}
	else
	{
		dstWidth = 240 ;
		dstHeight = 320 ;
		if ( lastOrientation != PORTRAIT )
			MoveWindow ( hwndPicture2, picture2Left, pictureTop, dstWidth, dstHeight, TRUE ) ;
		lastOrientation = PORTRAIT ;
	}

	hdcPicture2 = GetDC ( hwndPicture2 ) ; 
    if ( hdcPicture2 == 0 )
    {
        MessageBox ( NULL, "No hdcPicture2", NULL, MB_OK ) ;
		rtc = 7 ;
        goto quit ;
    }

	    // determine source area size

    WORD nBytesPerPixel = bmiImage1.biBitCount/8 ; // 3 bytes for 24 bit RGB, 1 byte for 8 bit grayscale
    unsigned char blue = 0 ;
	unsigned char green = 0 ;
	unsigned char red = 0 ;

	    // determine scale factors

	double hScale = (double) srcWidth / dstWidth ;
	double vScale = (double) srcHeight / dstHeight ;

	    // erase the picture

//	HBRUSH hbrBlack = (HBRUSH)GetStockObject(BLACK_BRUSH) ; 
//	RECT rectPicture2 ;
//	SetRect( &rectPicture2, 0, 0, dstWidth, dstHeight ) ;
//	FillRect ( hdcPicture2, &rectPicture2, hbrBlack ) ;

	long hCrossHair1 = dstHeight/2 - 2 ;
	long hCrossHair2 = dstHeight/2 + 2 ;
	long vCrossHair1 = dstWidth/2 - 2 ;
	long vCrossHair2 = dstWidth/2 + 2 ;

   	for ( LONG dstV=0 ; dstV<dstHeight ; dstV++ )
	{
            // don't overwrite horizontal crosshair

//		 if ( ( dstV == hCrossHair1 ) || ( dstV == hCrossHair2 ) ) continue ;

		    // draw horizontal crosshair pair of lines

//		if ( ( dstV == dstHeight/2-2 ) || ( dstV == dstHeight/2+2 ) )
//		{
//            SelectObject ( hdcPicture2, GetStockObject(WHITE_PEN) ) ; 

//            MoveToEx ( hdcPicture2, 0, dstHeight/2-2, NULL ) ;
//            LineTo ( hdcPicture2, dstWidth-1, dstHeight/2-2 ) ;

//            MoveToEx ( hdcPicture2, 0, dstHeight/2+2, NULL ) ;
//            LineTo ( hdcPicture2, dstWidth-1, dstHeight/2+2 ) ;

//			continue ; // don't copy picture over crosshair 
//		}

		for ( LONG dstH=0 ; dstH<dstWidth ; dstH++ )
		{
			COLORREF dstColor ;
			    
			if ( ( dstH == vCrossHair1 ) || ( dstH == vCrossHair2 ) 
				|| ( dstV == hCrossHair1 ) || ( dstV == hCrossHair2 ) )
			{       // create crosshairs
                dstColor = rgbGray ;
			}
			else
			{       // fetch color from source picture  

			    long srcH = dstH * hScale ;
			    long srcV = dstV * vScale ;
			
			        // calc address of source pixel

                unsigned char *pSrcPixel = pImage1 + ( srcH + (srcHeight-1-srcV)*srcWidth ) * nBytesPerPixel ;

			        // fetch the pixel

			    if ( nBytesPerPixel == 3 )  //  3 bytes for 24 bit RGB 
			    {  
			        blue = *pSrcPixel++ ;
			        green = *pSrcPixel++ ;
			        red = *pSrcPixel ;
			    }
			    else if ( nBytesPerPixel == 1 )  //  1 byte for 8 bit grayscale
			    {
				    blue = *pSrcPixel ;
				    green = blue ;
				    red = blue ;
			    }

				    // show on display

			    dstColor = RGB(red,green,blue) ;
		    }

			   // set the pixel on the destination
			SetPixel ( hdcPicture2, dstH, dstV, dstColor ) ;
		}
	}

	    // draw horizontal crosshair pair

//	SelectObject ( hdcPicture2, GetStockObject(WHITE_PEN) ) ; 

//	MoveToEx ( hdcPicture2, 0, hCrossHair1, NULL ) ;
//	LineTo ( hdcPicture2, dstWidth-1, hCrossHair1 ) ;

//	MoveToEx ( hdcPicture2, 0, hCrossHair2, NULL ) ;
//	LineTo ( hdcPicture2, dstWidth-1, hCrossHair2 ) ;
	
quit:
	if ( hdcPicture2 != 0 )
	{
        ReleaseDC ( hwndPicture2, hdcPicture2 ) ;
	    hdcPicture2 = 0 ;
	}

    return(rtc) ;
}

int AnalyzeImage ( void )
{
    int rtc = 0 ;

	LONG srcWidth = bmiImage1.biWidth ;
	LONG srcHeight = bmiImage1.biHeight ;
	LONG srcStep = pScan3dDlg->sampleInterval ;
    int nBitsPerPixel = bmiImage1.biBitCount ;

	    // check for image formats that I can work with

	if ( ( nBitsPerPixel != 24 ) && ( nBitsPerPixel != 8 ) )
	{
		rtc = 1 ;
		goto quit ;
	}
	WORD nBytesPerPixel = bmiImage1.biBitCount/8 ; // 3 bytes for 24 bit RGB, 1 byte for 8 bit grayscale

		// fetch current XYZ position from DRO

	double droX = GetDRO(800) ;
	double droY = GetDRO(801) ;
	double droZ = GetDRO(802) ;

	    // examine image and save hotspot for each vertical column

    nSamples = 0 ;  //  erase record of previous samples
	for ( long srcH=0 ; srcH<srcWidth ; srcH += srcStep )
	{
	    unsigned char blue ;
	    unsigned char green ;
	    unsigned char red ;
 		unsigned int sumGray ; 
		unsigned int saveGray =	pScan3dDlg->videoThreshold ;  //  min value worth logging pixel

		    //  examine vertical line for hot spot

		int nSamplesInit = nSamples ;    //  note number of samples before this column
        for ( long srcV=0 ; srcV<srcHeight ; srcV++ ) 
		{
			    // calc address of source pixel

            unsigned char *pSrcPixel = pImage1 + ( srcH + (srcHeight-1-srcV)*srcWidth ) * nBytesPerPixel ;

			    // fetch the pixel

			if ( nBytesPerPixel == 3 )  //  3 bytes for 24 bit RGB 
			{  
			    blue = *pSrcPixel++ ;
			    green = *pSrcPixel++ ;
			    red = *pSrcPixel ;
			}
			else if ( nBytesPerPixel == 1 )  //  1 byte 3 times for 8 bit grayscale
			{
				blue = *pSrcPixel ;
				green = blue ;
				red = blue ;
			}
			sumGray = (unsigned int) red + green + blue ;

			    // skip values lower than the ones saved thus far

			if ( sumGray < saveGray )    
				continue ;

			    // delete saved pixels if higher value found

            if ( sumGray > saveGray )  //  save brightest pixels
			{
			    nSamples = nSamplesInit ; // erase record of lesser samples
				saveGray = sumGray ; // note new high value
			}		
		        // this pixel is a keeper

			    //  make space for more samples if necessary

    		if ( nSamples == maxSamples )
       		{
    			unsigned long maxSamplesNeeded = maxSamples + 16 ;
    			SAMPLE *pRealloc = (SAMPLE*) realloc ( pSamples, maxSamplesNeeded*sizeof(SAMPLE) ) ;
    			if ( pRealloc == NULL )
    			{
    				char tmp[81] ;
    				sprintf ( tmp, "Insufficient memory for %li samples", (long)maxSamplesNeeded ) ;
    				MessageBox ( NULL, tmp, NULL, MB_OK ) ;
    				rtc = 3 ;
    				goto quit ;
    			} 
    			pSamples = pRealloc ;  //  update pointer to expanded memory
			    maxSamples = maxSamplesNeeded ;
		    }

			    // log this sample

		    SAMPLE *pSave = &pSamples[nSamples] ;  //  address to log this sample
		    pSave->h = srcH ;
		    pSave->v = srcV ;
		    pSave->red = red ;
		    pSave->green = green ;
		    pSave->blue = blue ;
		    pSave->gray = sumGray ;
		    pSave->droX = droX ;
		    pSave->droY = droY ;
		    pSave->droZ = droZ ;
		    nSamples++ ;  //  count this sample
		}
	}

quit:
//char tmp[81] ;
//sprintf ( tmp, "nSamples=%li    maxSamples=%li", (long)nSamples, (long)maxSamples ) ;
//DbgMsg((tmp)); 
	return(rtc) ;
}

int GraphShape ( void )
{
    int rtc = 0 ;
	HDC hdcPicture3 = 0 ;
	const int UNKNOWN = 0 ;
	const int PORTRAIT = 1 ;
	const int LANDSCAPE = 2 ;
	static int lastOrientation = UNKNOWN ;
	long dstWidth=0, dstHeight=0 ;  // dimensions of picture area

    HWND hwndPicture3 = GetDlgItem ( hScan3dDlg, IDC_PICTURE3 ) ;
    if ( hwndPicture3 == 0 )
    {
        MessageBox ( NULL, "No hwndPicture3", NULL, MB_OK ) ;
        rtc = 6 ;
		goto quit ;
    }

		// determine best output format and change if necessary

	LONG srcWidth = bmiImage1.biWidth ;
	LONG srcHeight = bmiImage1.biHeight ;
	if ( srcWidth > srcHeight )
	{
		dstWidth = 320 ;
		dstHeight = 240 ;
		if ( lastOrientation != LANDSCAPE )
			MoveWindow ( hwndPicture3, picture3Left, pictureTop, dstWidth, dstHeight, TRUE ) ;
		lastOrientation = LANDSCAPE ;
	}
	else
	{
		dstWidth = 240 ;
		dstHeight = 320 ;
		if ( lastOrientation != PORTRAIT )
			MoveWindow ( hwndPicture3, picture3Left, pictureTop, dstWidth, dstHeight, TRUE ) ;
		lastOrientation = PORTRAIT ;
	}

	hdcPicture3 = GetDC ( hwndPicture3 ) ; 
    if ( hdcPicture3 == 0 )
    {
        MessageBox ( NULL, "No hdcPicture3", NULL, MB_OK ) ;
		rtc = 7 ;
        goto quit ;
    }

	    // determine scale factors

	double hScale = (double) dstWidth / srcWidth ;
	double vScale = (double) dstHeight / srcHeight ;

	    // erase the background with black

    HBRUSH hbrBlack = (HBRUSH)GetStockObject(BLACK_BRUSH) ; 
	RECT rectPicture3 ;
	SetRect( &rectPicture3, 0, 0, dstWidth, dstHeight ) ;
	FillRect ( hdcPicture3, &rectPicture3, hbrBlack ) ;

	    // draw the graph

	for ( long iSample=0 ; iSample<nSamples ; iSample++ )
	{
		long dstH = pSamples[iSample].h * hScale ;
		long dstV = pSamples[iSample].v * vScale ;
        SetPixel ( hdcPicture3, dstH, dstV, rgbWhite ) ; 
	}

quit:
	if ( hdcPicture3 != 0 )
	{
        ReleaseDC ( hwndPicture3, hdcPicture3 ) ;
	    hdcPicture3 = 0 ;
	}
	return(rtc) ;
}

int GraphIntensities ( void )
{
    int rtc = 0 ;
	HDC hdcPicture4 = 0 ;
	const int UNKNOWN = 0 ;
	const int PORTRAIT = 1 ;
	const int LANDSCAPE = 2 ;
	static int lastOrientation = UNKNOWN ;
	long dstWidth=320, dstHeight=240 ;  // dimensions of picture area

	HWND hwndPicture4 = GetDlgItem ( hScan3dDlg, IDC_PICTURE4 ) ;
    if ( hwndPicture4 == 0 )
    {
        MessageBox ( NULL, "No hwndPicture4", NULL, MB_OK ) ;
        rtc = 6 ;
		goto quit ;
    }

		// determine best output format and change if necessary

	long srcWidth = bmiImage1.biWidth ;
	long srcHeight = 3*256 ;  //  allow for max sum of three unsigned bytes

    if ( srcWidth > bmiImage1.biHeight )
	{
		dstWidth = 320 ;  //  same width as picture3
		dstHeight = 256 ;
		if ( lastOrientation != LANDSCAPE )
			MoveWindow ( hwndPicture4, picture4Left, pictureTop, dstWidth, dstHeight, TRUE ) ;
		lastOrientation = LANDSCAPE ;
	}
	else
	{
		dstWidth = 240 ;  //  same width as picture3
		dstHeight = 256 ;
		if ( lastOrientation != PORTRAIT )
			MoveWindow ( hwndPicture4, picture4Left, pictureTop, dstWidth, dstHeight, TRUE ) ;
		lastOrientation = PORTRAIT ;
	}

	hdcPicture4 = GetDC ( hwndPicture4 ) ; 
    if ( hdcPicture4 == 0 )
    {
        MessageBox ( NULL, "No hdcPicture4", NULL, MB_OK ) ;
		rtc = 7 ;
        goto quit ;
    }

	    // determine scale factors

	double hScale = (double) dstWidth / srcWidth ;
	double vScale = (double) dstHeight / srcHeight ;

	    // erase the background with black

    HBRUSH hbrBlack = (HBRUSH)GetStockObject(BLACK_BRUSH) ; 
	HBRUSH hbrWhite = (HBRUSH)GetStockObject(WHITE_BRUSH) ;
	RECT rectPicture4 ;
	SetRect( &rectPicture4, 0, 0, dstWidth, dstHeight ) ;
	FillRect ( hdcPicture4, &rectPicture4, hbrWhite ) ;

	    // draw the threshold

	long threshV = dstHeight - 1 - pScan3dDlg->videoThreshold * vScale ;
	SelectObject ( hdcPicture4, GetStockObject(BLACK_PEN) ) ;
	MoveToEx ( hdcPicture4, 0, threshV, NULL ) ;  
    LineTo ( hdcPicture4, dstWidth-1, threshV ) ;

	    // draw the peak ref for colors as dashed line

	HPEN hpenBlackDotted = CreatePen ( PS_DOT, 0, rgbBlack ) ;
	SelectObject ( hdcPicture4, hpenBlackDotted ) ;
	long dstV = dstHeight - 1 - 255*vScale ;
	MoveToEx ( hdcPicture4, 0, dstV, NULL ) ;  
    LineTo ( hdcPicture4, dstWidth-1, dstV ) ;
	SelectObject ( hdcPicture4, GetStockObject(BLACK_PEN) ) ;
    DeleteObject ( hpenBlackDotted ) ;

	    // draw the color graphs

	    // draw graph for net gray points

	SelectObject ( hdcPicture4, GetStockObject(BLACK_PEN) ) ;
	for ( long iSample=0 ; iSample<nSamples ; iSample++ )
	{
		long dstH = pSamples[iSample].h * hScale ;
		long dstV = dstHeight - 1 - pSamples[iSample].gray * vScale ;
        if ( iSample == 0 )
			MoveToEx ( hdcPicture4, dstH, dstV, NULL ) ;  //  start the graph 
		LineTo ( hdcPicture4, dstH, dstV ) ;
	}

	    // draw graph for red component of points

    HPEN hpenRed = CreatePen ( PS_SOLID, 0, rgbRed ) ;
	SelectObject ( hdcPicture4, hpenRed ) ;
	for ( long iSample=0 ; iSample<nSamples ; iSample++ )
	{
		long dstH = pSamples[iSample].h * hScale ;
		long dstV = dstHeight - 1 - pSamples[iSample].red * vScale ;
        if ( iSample == 0 )
			MoveToEx ( hdcPicture4, dstH, dstV, NULL ) ;  //  start the graph 
		LineTo ( hdcPicture4, dstH, dstV ) ;
	}
    SelectObject ( hdcPicture4, GetStockObject(BLACK_PEN) ) ;
    DeleteObject ( hpenRed ) ;

	    // draw graph for green component of points

    HPEN hpenGreen = CreatePen ( PS_SOLID, 0, rgbGreen ) ;
	SelectObject ( hdcPicture4, hpenGreen ) ;
 	for ( long iSample=0 ; iSample<nSamples ; iSample++ )
	{
		long dstH = pSamples[iSample].h * hScale ;
		long dstV = dstHeight - 1 - pSamples[iSample].green * vScale ;
        if ( iSample == 0 )
			MoveToEx ( hdcPicture4, dstH, dstV, NULL ) ;  //  start the graph 
		LineTo ( hdcPicture4, dstH, dstV ) ;
	}
    SelectObject ( hdcPicture4, GetStockObject(BLACK_PEN) ) ;
    DeleteObject ( hpenGreen ) ;

	    // draw graph for blue component of points

    HPEN hpenBlue = CreatePen ( PS_SOLID, 0, rgbBlue ) ;
	SelectObject ( hdcPicture4, hpenBlue ) ;
	for ( long iSample=0 ; iSample<nSamples ; iSample++ )
	{
		long dstH = pSamples[iSample].h * hScale ;
		long dstV = dstHeight - 1 - pSamples[iSample].blue * vScale ;
        if ( iSample == 0 )
			MoveToEx ( hdcPicture4, dstH, dstV, NULL ) ;  //  start the graph 
		LineTo ( hdcPicture4, dstH, dstV ) ;
	}
	SelectObject ( hdcPicture4, GetStockObject(BLACK_PEN) ) ;
    DeleteObject ( hpenBlue ) ;


//	for ( long iSample=0 ; iSample<nSamples ; iSample++ )
//	{
//		long dstH = pSamples[iSample].h * hScale ;
//		long dstV ;
//
//		dstV = dstHeight - 1 - pSamples[iSample].red * vScale ;
//        SetPixel ( hdcPicture4, dstH, dstV, rgbRed ) ; 
//
//		dstV = dstHeight - 1 - pSamples[iSample].green * vScale ;
//        SetPixel ( hdcPicture4, dstH, dstV, rgbGreen ) ; 
//
//		dstV = dstHeight - 1 - pSamples[iSample].blue * vScale ;
//        SetPixel ( hdcPicture4, dstH, dstV, rgbBlue ) ; 
//		
//		dstV = dstHeight - 1 - pSamples[iSample].gray * vScale ;
//        SetPixel ( hdcPicture4, dstH, dstV, rgbBlack ) ; 
//	}

quit:
	if ( hdcPicture4 != 0 )
	{
        ReleaseDC ( hwndPicture4, hdcPicture4 ) ;
	    hdcPicture4 = 0 ;
	}
	return(rtc) ;
}

int UpdateScan3d ( void )
{
	int rtc = 0 ; // return error code
	static int fIsWorking = FALSE ;	// entry counter

	if ( fIsScan3dOpen == FALSE )
		return (1) ;

	if ( fIsWorking == TRUE ) return(2) ;  // prevent recursion
	    fIsWorking = TRUE ;  // flag as work in progress

	if ( pScan3dDlg->fIsPauseVideo == TRUE ) // skip update if paused
	{
//DbgMsg(("UpdateScan3d() Paused"));
		rtc = 3 ;
		goto quit ;
	}

	if ( hwndVideoRect1 == 0 )  // must have a video window
	{
//DbgMsg(("UpdateScan3d() hwndVideoRect1 == 0"));
        rtc = 4 ;
		goto quit ;
	}

	if ( !capGrabFrame ( hwndVideoRect1 ) )  // snapshot one frame and show it
	{
//DbgMsg(("UpdateScan3d() capGrabFrame() failed"));
        rtc = 5 ;
		goto quit ;
	}
//DbgMsg(("Frame Grabbed in UpdateScane3d()"));

	FetchImage() ;
	OrientImage() ;
	ShowImage() ;
	AnalyzeImage() ;
	GraphShape() ; 
	GraphIntensities() ;

quit:
	fIsWorking = FALSE ; // flag as finished
	return (rtc) ;
}  

int CloseScan3d ( void )
{
DbgMsg(("CloseScan3d() entry"));
	int rtc = 0 ; 

	if ( pScan3dDlg != NULL )
	{
		pScan3dDlg->DestroyWindow() ;
DbgMsg(("CloseScan3d() check6"));	
		delete pScan3dDlg ; // free memory from heap
DbgMsg(("CloseScan3d() check7"));	
        pScan3dDlg = NULL ;
		cwndScan3dDlgParent.Detach() ; // detach from main window
	}

DbgMsg(("CloseScan3d() exit"));	
	return (rtc) ;
}

int DestroyVideoWindow ( void ) 
{
DbgMsg(("DestroyVideoWindow() entry"));
    int rtc = 0 ;

	StopTimer() ;
DbgMsg(("DestroyVideoWindow() Timer stopped"));

    if ( pSamples != NULL )  //  free memory used for samples
	{
        free ( pSamples ) ;
		pSamples = NULL ;
		maxSamples = 0 ;
		nSamples = 0 ;
	}

	if ( pImage1 != NULL )  //  free memory used for image capture
	{
DbgMsg(("DestroyVideoWindow() freeing image capture memory"));
        free ( pImage1 ) ;
		pImage1 = NULL ;
	}

DbgMsg(("DestroyVideoWindow() check2"));	
    if ( hwndVideoRect1 )
	{
DbgMsg(("DestroyVideoWindow() check3"));	
		capDriverDisconnect ( hwndVideoRect1 ) ;
DbgMsg(("DestroyVideoWindow() check4"));	
		DestroyWindow ( hwndVideoRect1 ) ;
DbgMsg(("DestroyVideoWindow() check5"));	
        hwndVideoRect1 = 0 ;
DbgMsg(("DestroyVideoWindow() check6"));	
	}
DbgMsg(("DestroyVideoWindow() check7"));
quit:
DbgMsg(("DestroyVideoWindow() exit"));
	return(rtc);
}

//---------------------------------------

int ActivateVideoSourceDlg ( void )
{
	DbgMsg(("ActivateVideoSourceDlg() entry"));

	int rtc = 0 ; // return error code    
	
	// get video capabilities  

	if (!hwndVideoRect1) // must have video window
	{
        MessageBox ( NULL, "ActivateVideoSourceDlg()\n\nNo video window", NULL, MB_OK ) ;
        rtc = 1 ;
		goto quit ;
	}
   
	CAPDRIVERCAPS psCaps ; // structure for camera driver capabilities
	if (!capDriverGetCaps ( hwndVideoRect1, &psCaps, sizeof(psCaps) ) ) // must be able to get capabilities
	{
        MessageBox ( NULL, "ActivateVideoSourceDlg()\n\nFailed to get video capabilities", NULL, MB_OK ) ;
        rtc = 3 ;
		goto quit ;
	}

	if ( psCaps.fHasDlgVideoSource == FALSE ) // must have source capability
	{
        MessageBox ( NULL, "ActivateVideoSourceDlg()\n\nNo source capability", NULL, MB_OK ) ;
        rtc = 4 ;
		goto quit ;
	}

	if (!capDlgVideoSource(hwndVideoRect1)) // must succeed at getting source dlg
	{
        MessageBox ( NULL, "ActivateVideoSourceDlg()\n\nSource dialog failed", NULL, MB_OK ) ;
        rtc = 5 ;
		goto quit ;
	}
    
quit:
	DbgMsg(("ActivateVideoSourceDlg() exit"));
	return(rtc) ;
}

int ActivateVideoFormatDlg ( void )
{
	DbgMsg(("ActivateVideoFormatDlg() entry"));

	int rtc = 0 ; // return error code    
	
	// get video capabilities  

	if (!hwndVideoRect1) // must have video window
	{
        MessageBox ( NULL, "ActivateVideoFormatDlg()\n\nNo video window", NULL, MB_OK ) ;
		rtc = 1 ;
		goto quit ;
	}
   
	CAPDRIVERCAPS psCaps ;  // structure for video driver capabilities
	if (!capDriverGetCaps ( hwndVideoRect1, &psCaps, sizeof(psCaps) ) ) // must be able to get capabilities
	{
        MessageBox ( NULL, "ActivateVideoFormatDlg()\n\nFailed to get video capabilities", NULL, MB_OK ) ;
        rtc = 3 ;
		goto quit ;
	}

	if ( psCaps.fHasDlgVideoFormat == FALSE ) // must have format capability
	{
        MessageBox ( NULL, "ActivateVideoFormatDlg()\n\nNo format capability", NULL, MB_OK ) ;
        rtc = 4 ;
		goto quit ;
	}

	if (!capDlgVideoFormat(hwndVideoRect1)) // must succeed at getting format dlg
	{
        MessageBox ( NULL, "ActivateVideoFormatDlg()\n\nFormat dialog failed", NULL, MB_OK ) ;
        rtc = 5 ;
		goto quit ;
	}
    
quit:
	DbgMsg(("ActivateVideoFormatDlg() exit"));
	return(rtc) ;
} 

int ActivateVideoDisplayDlg ( void )
{
	DbgMsg(("ActivateVideoDisplayDlg() entry"));

	int rtc = 0 ; // return error code    
	
	// get video capabilities  

	if (!hwndVideoRect1) // must have video window
	{
        MessageBox ( NULL, "ActivateVideoDisplayDlg()\n\nNo video window", NULL, MB_OK ) ;
		rtc = 1 ;
		goto quit ;
	}
   
	CAPDRIVERCAPS psCaps ;  // structure for video driver capabilities
	if (!capDriverGetCaps ( hwndVideoRect1, &psCaps, sizeof(psCaps) ) ) // must be able to get capabilities
	{
        MessageBox ( NULL, "ActivateVideoDisplayDlg()\n\nFailed to get video capabilities", NULL, MB_OK ) ;
        rtc = 3 ;
		goto quit ;
	}

	if ( psCaps.fHasDlgVideoDisplay == FALSE ) // must have format capability
	{
        MessageBox ( NULL, "ActivateVideoDisplayDlg()\n\nNo display capability", NULL, MB_OK ) ;
        rtc = 4 ;
		goto quit ;
	}

	if (!capDlgVideoDisplay(hwndVideoRect1)) // must succeed at getting format dlg
	{
        MessageBox ( NULL, "ActivateVideoDisplayDlg()\n\nDisplay dialog failed", NULL, MB_OK ) ;
        rtc = 5 ;
		goto quit ;
	}
    
quit:
	DbgMsg(("ActivateVideoDisplayDlg() exit"));
	return(rtc) ;
} 


int SnapshotToClipboard ( void )
{
	int rtc = 0 ;

	if ( !capEditCopy ( hwndVideoRect1 ) )  // save snapshot to clipboard
	{
        rtc = 6 ;
		MessageBox ( NULL, "SnapshotToClipboard() capEditCopy(hwndVideoRect1) failed", NULL, MB_OK ) ;
		goto quit ;
	}
    DbgMsg(("SnapshotToClipboard() Frame copied to clipboard"));
quit:
	return(rtc) ;
}


int DrawTest1 ( void ) 
{
DbgMsg(("DrawTest1() entry"));
	int rtc = 0 ;

//    PAINTSTRUCT ps1 ;
//    PAINTSTRUCT ps2 ;
//	PAINTSTRUCT ps3 ;
	HDC hdcVideoRect1 = 0 ;
	HDC hdcPicture2 = 0 ;
	HDC hdcPicture3 = 0 ;

//    hdcVideoRect1 = BeginPaint ( hwndVideoRect1, &ps1 ) ;
    hdcVideoRect1 = GetDC ( hwndVideoRect1 ) ;
    if ( hdcVideoRect1 == 0 )
    {
        MessageBox ( NULL, "No hdcVideoRect1", NULL, MB_OK ) ;
        rtc = 1 ;
		goto quit ;
    }

    HWND hwndPicture2 = GetDlgItem ( hScan3dDlg, IDC_PICTURE2 ) ;
    if ( hwndPicture2 == 0 )
    {
        MessageBox ( NULL, "No hwndPicture2", NULL, MB_OK ) ;
        rtc = 2 ;
		goto quit ;
    }

//	MoveWindow ( hwndPicture2, 320, 30, 320, 240, TRUE ) ;
    hdcPicture2 = GetDC ( hwndPicture2 ) ; 
    if ( hdcPicture2 == 0 )
    {
        MessageBox ( NULL, "No hdcPicture2", NULL, MB_OK ) ;
		rtc = 3 ;
        goto quit ;
    }

    HWND hwndPicture3 = GetDlgItem ( hScan3dDlg, IDC_PICTURE3 ) ;
    if ( hwndPicture3 == 0 )
    {
        MessageBox ( NULL, "No hwndPicture3", NULL, MB_OK ) ;
		rtc = 4 ;
        goto quit ;
    }
//	MoveWindow ( hwndPicture3, 670, 30, 320, 240, TRUE ) ;
//    hdcPicture3 = BeginPaint ( hwndPicture3, &ps3 ) ;
    hdcPicture3 = GetDC ( hwndPicture3 ) ;
	if ( hdcPicture3 == 0 )
    {
        MessageBox ( NULL, "No hdcPicture3", NULL, MB_OK ) ;
		rtc = 5 ;
        goto quit ;
    }

    MoveToEx ( hdcPicture2, 50, 100, NULL ) ;
    LineTo ( hdcPicture2, 50, 50 ) ;

    for ( int x=0 ; x < 50 ; x++ )
    {
        int y = 2*x ;
	    SetPixel (hdcPicture3, x, y, rgbBlack ) ;
    }

    MoveToEx ( hdcPicture3, 0, 0, NULL ) ;
    LineTo ( hdcPicture3, 50, 50 ) ;

	for ( x=0 ; x<320 ; x++ )
	{
		for ( int y=0 ; y<240 ; y++ )
		{
            COLORREF rgbColor = GetPixel ( hdcVideoRect1, x, y ) ;
			SetPixel ( hdcPicture3, x, y, rgbColor ) ;
		}
	}

//    BitBlt ( hdcPicture3, 0, 0, 100, 100, hdcPicture2, 0, 0, SRCCOPY ) ;
//    BitBlt ( hdcPicture2, 0, 0, 100, 100, hdcVideoRect1, 0, 0, SRCCOPY ) ;

RECT rect1, rect2, rect3 ;
GetClientRect ( hwndVideoRect1, &rect1 ) ;
GetClientRect ( hwndPicture2, &rect2 ) ;
GetClientRect ( hwndPicture3, &rect3 ) ;

//BitBlt( hdcPicture3, 0, 0, rect3.right, rect3.bottom, hdcVideoRect1, 0, 0, SRCCOPY ) ;
//StretchBlt ( hdcPicture2, 0, 0, rect2.right, rect2.bottom, hdcVideoRect1, 0, 0, rect1.right, rect1.bottom, SRCCOPY ) ; 

char tmp2[255] ;
sprintf ( tmp2, "dst = %ix%i\n\nsrc = %ix%i", rect3.right, rect3.bottom, rect1.right, rect1.bottom ) ;
MessageBox ( NULL, tmp2, NULL, MB_OK ) ;

quit:
	if ( hdcPicture3 )
	{
//		ValidateRect ( hwndPicture3, NULL ) ;
//      EndPaint ( hwndPicture3, &ps3 ) ;
		ReleaseDC ( hwndPicture3, hdcPicture3 ) ;
		hdcPicture3 = 0 ;
DbgMsg(("Picture3 finished"));
	}

	if ( hdcPicture2 )
	{
//       	EndPaint ( hwndPicture2, &ps2 ) ;
	    ReleaseDC ( hwndPicture2, hdcPicture2 ) ;	 
		hdcPicture2 = 0 ;
	}

	if ( hdcVideoRect1 )
	{
//        	EndPaint ( hwndVideoRect1, &ps1 ) ;
	    ReleaseDC ( hwndVideoRect1, hdcVideoRect1 ) ;
		hdcVideoRect1 = 0 ;
	}
char tmp1[81] ;
sprintf ( tmp1, "DrawTest1() exit    rtc=%i", rtc ) ;
DbgMsg((tmp1));
	return(rtc) ;
}

int SaveScan3dData ( char* iniFileName ) 
{  
    int rtc = 0 ;
	char* Section = "Scan3d" ; 
	char tmp[255] ;

			// create file name with Mach3 directory for path
	
	extern CString DefDir ; // default directory
    char iniFullName[255] ;
	sprintf ( iniFullName, "%s\\%s", DefDir, iniFileName ) ;

	    // save the data

	sprintf ( tmp, "%i", pScan3dDlg->xOrder ) ;
	WritePrivateProfileString ( Section, "xOrder", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->xFirst ) ;
	WritePrivateProfileString ( Section, "xFirst", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->xLast ) ;
	WritePrivateProfileString ( Section, "xLast", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->xStep ) ;
	WritePrivateProfileString ( Section, "xStep", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->xScale ) ;
	WritePrivateProfileString ( Section, "xScale", tmp, iniFullName ) ;


	sprintf ( tmp, "%i", pScan3dDlg->yOrder ) ;
	WritePrivateProfileString ( Section, "yOrder", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->yFirst ) ;
	WritePrivateProfileString ( Section, "yFirst", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->yLast ) ;
	WritePrivateProfileString ( Section, "yLast", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->yStep ) ;
	WritePrivateProfileString ( Section, "yStep", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->yScale ) ;
	WritePrivateProfileString ( Section, "yScale", tmp, iniFullName ) ;


	sprintf ( tmp, "%i", pScan3dDlg->zOrder ) ;
	WritePrivateProfileString ( Section, "zOrder", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->zFirst ) ;
	WritePrivateProfileString ( Section, "zFirst", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->zLast ) ;
	WritePrivateProfileString ( Section, "zLast", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->zStep ) ;
	WritePrivateProfileString ( Section, "zStep", tmp, iniFullName ) ;

	sprintf ( tmp, "%lf", pScan3dDlg->zScale ) ;
	WritePrivateProfileString ( Section, "zScale", tmp, iniFullName ) ;


	sprintf ( tmp, "%lf", pScan3dDlg->feed ) ;
	WritePrivateProfileString ( Section, "feed", tmp, iniFullName ) ;


	sprintf ( tmp, "%i", pScan3dDlg->videoThreshold ) ;
	WritePrivateProfileString ( Section, "videoThreshold", tmp, iniFullName ) ;

	sprintf ( tmp, "%i", pScan3dDlg->sampleInterval ) ;
	WritePrivateProfileString ( Section, "sampleInterval", tmp, iniFullName ) ;

    sprintf ( tmp, "%i", pScan3dDlg->videoRotation ) ;
	WritePrivateProfileString ( Section, "videoRotation", tmp, iniFullName ) ;

	sprintf ( tmp, "%i", pScan3dDlg->videoMirror ) ;
	WritePrivateProfileString ( Section, "videoMirror", tmp, iniFullName ) ;


	sprintf ( tmp, "%lg", A[0] ) ;
	WritePrivateProfileString ( Section, "A[0]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", A[1] ) ;
	WritePrivateProfileString ( Section, "A[1]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", A[2] ) ;
	WritePrivateProfileString ( Section, "A[2]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", A[3] ) ;
	WritePrivateProfileString ( Section, "A[3]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", A[4] ) ;
	WritePrivateProfileString ( Section, "A[4]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", A[5] ) ;
	WritePrivateProfileString ( Section, "A[5]", tmp, iniFullName ) ;


	sprintf ( tmp, "%lg", B[0] ) ;
	WritePrivateProfileString ( Section, "B[0]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", B[1] ) ;
	WritePrivateProfileString ( Section, "B[1]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", B[2] ) ;
	WritePrivateProfileString ( Section, "B[2]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", B[3] ) ;
	WritePrivateProfileString ( Section, "B[3]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", B[4] ) ;
	WritePrivateProfileString ( Section, "B[4]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", B[5] ) ;
	WritePrivateProfileString ( Section, "A[5]", tmp, iniFullName ) ;


	sprintf ( tmp, "%lg", C[0] ) ;
	WritePrivateProfileString ( Section, "C[0]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", C[1] ) ;
	WritePrivateProfileString ( Section, "C[1]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", C[2] ) ;
	WritePrivateProfileString ( Section, "C[2]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", C[3] ) ;
	WritePrivateProfileString ( Section, "C[3]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", C[4] ) ;
	WritePrivateProfileString ( Section, "C[4]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", C[5] ) ;
	WritePrivateProfileString ( Section, "C[5]", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", pScan3dDlg->hfov ) ;
	WritePrivateProfileString ( Section, "hfov", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", pScan3dDlg->vfov ) ;
	WritePrivateProfileString ( Section, "vfov", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", pScan3dDlg->elevation ) ;
	WritePrivateProfileString ( Section, "elevation", tmp, iniFullName ) ;

	sprintf ( tmp, "%lg", pScan3dDlg->azimuth ) ;
	WritePrivateProfileString ( Section, "azimuth", tmp, iniFullName ) ;

quit:
	return(rtc) ;
}

int LoadScan3dData ( char* iniFileName ) 
{  
    int rtc = 0 ;
	char* Section = "Scan3d" ; 
	char tmp[255] ;

		// create file name with Mach3 directory for path
	
	extern CString DefDir ; // default directory
    char iniFullName[255] ;
	sprintf ( iniFullName, "%s\\%s", DefDir, iniFileName ) ;

		// fetch the data

	GetPrivateProfileString ( Section, "xOrder", "1", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->xOrder = atoi(tmp) ;

	GetPrivateProfileString ( Section, "xFirst", "-0.5", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->xFirst = atof(tmp) ;

	GetPrivateProfileString ( Section, "xLast", "+0.5", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->xLast = atof(tmp) ;

	GetPrivateProfileString ( Section, "xStep", "0.010", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->xStep = atof(tmp) ;

	GetPrivateProfileString ( Section, "xScale", "1.0", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->xScale = atof(tmp) ;


	GetPrivateProfileString ( Section, "yOrder", "2", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->yOrder = atoi(tmp) ;

	GetPrivateProfileString ( Section, "yFirst", "-0.5", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->yFirst = atof(tmp) ;

	GetPrivateProfileString ( Section, "yLast", "+0.5", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->yLast = atof(tmp) ;

	GetPrivateProfileString ( Section, "yStep", "1.0", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->yStep = atof(tmp) ;
    
	GetPrivateProfileString ( Section, "yScale", "1.0", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->yScale = atof(tmp) ;


	GetPrivateProfileString ( Section, "zOrder", "3", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->zOrder = atoi(tmp) ;

	GetPrivateProfileString ( Section, "zFirst", "-0.5", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->zFirst = atof(tmp) ;

	GetPrivateProfileString ( Section, "zLast", "+0.5", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->zLast = atof(tmp) ;

	GetPrivateProfileString ( Section, "zStep", "0.4", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->zStep = atof(tmp) ;

	GetPrivateProfileString ( Section, "zScale", "1.0", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->zScale = atof(tmp) ;


	GetPrivateProfileString ( Section, "feed", "5", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->feed = atof(tmp) ;


	GetPrivateProfileString ( Section, "videoThreshold", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->videoThreshold = atoi(tmp) ;

	GetPrivateProfileString ( Section, "sampleInterval", "1", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->sampleInterval = atoi(tmp) ;

	GetPrivateProfileString ( Section, "videoRotation", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->videoRotation = atoi(tmp) ;

	GetPrivateProfileString ( Section, "videoMirror", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->videoMirror = atoi(tmp) ;


	GetPrivateProfileString ( Section, "A[0]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    A[0] = atof(tmp) ;

	GetPrivateProfileString ( Section, "A[1]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    A[1] = atof(tmp) ;

	GetPrivateProfileString ( Section, "A[2]", "0.707", tmp, sizeof(tmp)-1, iniFullName ) ;
    A[2] = atof(tmp) ;

	GetPrivateProfileString ( Section, "A[3]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    A[3] = atof(tmp) ;

	GetPrivateProfileString ( Section, "A[4]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    A[4] = atof(tmp) ;

	GetPrivateProfileString ( Section, "A[5]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    A[5] = atof(tmp) ;


	GetPrivateProfileString ( Section, "B[0]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    B[0] = atof(tmp) ;

	GetPrivateProfileString ( Section, "B[1]", "1", tmp, sizeof(tmp)-1, iniFullName ) ;
    B[1] = atof(tmp) ;

	GetPrivateProfileString ( Section, "B[2]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    B[2] = atof(tmp) ;

	GetPrivateProfileString ( Section, "B[3]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    B[3] = atof(tmp) ;

	GetPrivateProfileString ( Section, "B[4]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    B[4] = atof(tmp) ;

	GetPrivateProfileString ( Section, "B[5]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    B[5] = atof(tmp) ;


	GetPrivateProfileString ( Section, "C[0]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    C[0] = atof(tmp) ;

	GetPrivateProfileString ( Section, "C[1]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    C[1] = atof(tmp) ;

	GetPrivateProfileString ( Section, "C[2]", "0.707", tmp, sizeof(tmp)-1, iniFullName ) ;
    C[2] = atof(tmp) ;

	GetPrivateProfileString ( Section, "C[3]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    C[3] = atof(tmp) ;

	GetPrivateProfileString ( Section, "C[4]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    C[4] = atof(tmp) ;

	GetPrivateProfileString ( Section, "C[5]", "0", tmp, sizeof(tmp)-1, iniFullName ) ;
    C[5] = atof(tmp) ;


	GetPrivateProfileString ( Section, "hfov", "1", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->hfov = atof(tmp) ;

	GetPrivateProfileString ( Section, "vfov", "1", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->vfov = atof(tmp) ;

	GetPrivateProfileString ( Section, "elevation", "1", tmp, sizeof(tmp)-1, iniFullName ) ;
    pScan3dDlg->elevation = atof(tmp) ;

	GetPrivateProfileString ( Section, "azimuth", "1", tmp, sizeof(tmp)-1, iniFileName ) ;
    pScan3dDlg->azimuth = atof(tmp) ;

quit:
	return(rtc) ;
}

int DoScan ( void )
{
	int rtc=0 ;
    FILE *outFile = NULL ;

	outFile = fopen ( "Scan3dCloud.xyz", "w" ) ;
	if ( outFile == NULL )
	{
        MessageBox ( NULL, "Unable to open file to save part points", NULL, MB_OK ) ;
		rtc = 6 ;
		goto quit ;
	}

	DoMotion ( PART, outFile ) ;

quit:
	if ( outFile != NULL )
	{
        fclose ( outFile ) ;
		outFile = NULL ;
	}

	return(rtc) ;    
}

int DoCal ( void )
{
	int rtc=0 ;
    FILE *outFile = NULL ;
	FILE *inFile = NULL ;
	char *calFileName = "Scan3dCal.pts" ; 

	    // collect calibration data

	outFile = fopen ( calFileName, "w" ) ;
	if ( outFile == NULL )
	{
        MessageBox ( NULL, "Unable to open file to save calibration points", NULL, MB_OK ) ;
		rtc = 6 ;
		goto quit ;
	}

	DoMotion ( CALIBRATION, outFile ) ;
	fclose ( outFile ) ;
	outFile = NULL ;

	    // calculate coefficients

	inFile = fopen ( calFileName, "r" ) ;
	if ( inFile == NULL )
	{
        MessageBox ( NULL, "Unable to open file to read calibration points", NULL, MB_OK ) ;
		rtc = 7 ;
		goto quit ;
	}

	CalcCoef ( inFile ) ;	// calculate the coefficients
	CalcFov() ;				// calculate the field of view 

    fclose ( inFile ) ;
	inFile = NULL ;

quit:
	if ( outFile != NULL )
	{
        fclose ( outFile ) ;
		outFile = NULL ;
	}

	if ( inFile != NULL )
	{
        fclose ( inFile ) ;
		inFile = NULL ;
	}

	return(rtc) ;
}

int DoMotion ( int scanType, FILE *outFile )
{
DbgMsg(("DoMotion() entry"));
    static BOOL fRunning = FALSE ;  //  prevent recursive entry 
    if ( fRunning == TRUE )
	{
DbgMsg(("DoMotion() recursive entry prevented"));
		return(1) ;
	}
	fRunning = TRUE ;

	StopTimer() ;  //  stop timed image update
	
    int rtc = 0 ;

	int nx = 1 ;
	int ny = 1 ;
	int nz = 1 ;

    int xOrder = pScan3dDlg->xOrder ;
	double xFirst = pScan3dDlg->xFirst ;
	double xLast = pScan3dDlg->xLast ;
	double xStep = pScan3dDlg->xStep ;

    int yOrder = pScan3dDlg->yOrder ;
	double yFirst = pScan3dDlg->yFirst ;
	double yLast = pScan3dDlg->yLast ;
	double yStep = pScan3dDlg->yStep ;

	int zOrder = pScan3dDlg->zOrder ;
	double zFirst = pScan3dDlg->zFirst ;
	double zLast = pScan3dDlg->zLast ;
	double zStep = pScan3dDlg->zStep ;

	if ( ( xOrder != 0 ) && ( xOrder == yOrder ) )
	{
		char tmp[81] ;
		sprintf ( tmp, "xOrder and yOrder both equal %i", xOrder ) ;
        MessageBox ( NULL, tmp, NULL, MB_OK ) ;
		rtc = 3 ;
		goto quit ;
	}

	if ( ( xOrder != 0 ) && ( xOrder == zOrder ) )
	{
		char tmp[81] ;
		sprintf ( tmp, "xOrder and zOrder both equal %i", xOrder ) ;
        MessageBox ( NULL, tmp, NULL, MB_OK ) ;
		rtc = 4 ;
		goto quit ;
	}

	if ( ( yOrder != 0 ) && ( yOrder == zOrder ) )
	{
		char tmp[81] ;
		sprintf ( tmp, "yOrder and zOrder both equal %i", yOrder ) ;
        MessageBox ( NULL, tmp, NULL, MB_OK ) ;
		rtc = 5 ;
		goto quit ;
	}

    if ( ( xOrder == 0 ) || ( xStep == 0 ) )
	{
        nx = 1 ;
		xStep = xLast - xFirst ;
	}
	else if ( xLast < xFirst )
	{
        xStep = -fabs(xStep) ;
        nx = 1 + ( xLast - xFirst ) / xStep ;
	}
	else
	{
		xStep = +fabs(xStep) ;
        nx = 1 + ( xLast - xFirst ) / xStep ;
	}

	if ( ( yOrder == 0 ) || ( yStep == 0 ) )
	{
        ny = 1 ;
		yStep = yLast - yFirst ;
	}
	else if ( yLast < yFirst )
	{
        yStep = -fabs(yStep) ;
        ny = (INT) ( 1 + ( yLast - yFirst ) / yStep ) ;
	}
	else
	{
		yStep = +fabs(yStep) ;
        ny = (int) ( 1 + ( yLast - yFirst ) / yStep ) ;
	}

	if ( ( zOrder == 0 ) || ( zStep == 0 ) )
	{
        nz = 1 ;
		zStep = zLast - zFirst ;
	}
	else if ( zLast < zFirst )
	{
        zStep = -fabs(zStep) ;
        nz = (int) ( 1 + ( zLast - zFirst ) / zStep ) ;
	}
	else
	{
		zStep = +fabs(zStep) ;
        nz =(int) ( 1 + ( zLast - zFirst ) / zStep ) ;
	}

		// initialize for ordered axis motion

	bool sendGcode = FALSE ;
	char *axisName[4] = { NULL, NULL, NULL, NULL } ;
	double axisValue[4] = { 0.0, 0.0, 0.0, 0.0 } ;
	int nSteps[4] = { 0, 1, 1, 1 } ;
	double first[4] = { 0.0, 0.0, 0.0, 0.0 } ;
	double last[4] = { 0.0, 0.0, 0.0, 0.0 } ;
	double step[4] = { 0.0, 0.0, 0.0, 0.0 } ;

	if ( xOrder != 0 )
	{
		sendGcode = TRUE ;
        axisName[xOrder] = "X " ;
		nSteps[xOrder] = nx ;
		first[xOrder] = xFirst ;
		last[xOrder] = xLast ;
		step[xOrder] = xStep ;
	}

	if ( yOrder != 0 )
	{
		sendGcode = TRUE ;
        axisName[yOrder] = "Y " ;
		nSteps[yOrder] = ny ;
		first[yOrder] = yFirst ;
		last[yOrder] = yLast ;
		step[yOrder] = yStep ;
	}

	if ( zOrder != 0 )
	{
		sendGcode = TRUE ;
        axisName[zOrder] = "Z " ;
		nSteps[zOrder] = nz ;
		first[zOrder] = zFirst ;
		last[zOrder] = zLast ;
		step[zOrder] = zStep ;
	}
	
char  tmp[81] ;
sprintf ( tmp, "nx=%i    ny=%i    nz=%i", nx, ny, nz ) ;
DbgMsg((tmp));

    for ( int k=0 ; k<nSteps[3] ; k++ )
    {    
        char block3[81] = "" ;	
		if ( axisName[3] != NULL )
		{
            axisValue[3] = first[3] + step[3] * k ;
            sprintf ( block3, "%s %lg ", axisName[3], axisValue[3] ) ;
		}

		for ( int j=0 ; j<nSteps[2] ; j++ )
	    {
			char block2[81] = "" ;
			if ( axisName[2] != NULL )
	    	{
		        axisValue[2] = first[2] + step[2] * j ;
				sprintf ( block2, "%s %lg ", axisName[2], axisValue[2] ) ;
		    }

			for ( int i=0 ; i<nSteps[1] ; i++ )
		    {
				char block1[81] = "" ;
				if ( axisName[1] != NULL )
				{  
     			    axisValue[1] = first[1] + step[1] * i ;
                    sprintf ( block1, "%s %lg ", axisName[1], axisValue[1] ) ;  
				}
 
                if ( sendGcode )
				{
			        char block[255] ;
			        sprintf ( block, "G01 %s %s %s F %lf", block1, block2, block3, pScan3dDlg->feed ) ;
				    Code ( block ) ;  //  send next line of Gcode to move to the next location
				}

					// wait for motion to stop

				for ( BOOL done = FALSE ; ; )
				{
					MSG msg ;
                    if ( GetMessage ( &msg, NULL, 0, 0 ) )
					{ 
					    TranslateMessage ( &msg ) ;
					    DispatchMessage ( &msg ) ;
					}

					if ( pScan3dDlg->fEnableScan == FALSE )
					    goto quit ;

					if ( GetLED(999) == FALSE )  //  continue to idle if in motion 
						break ;
				}

				UpdateScan3d() ;  // grab a frame and process the image

                if ( scanType == CALIBRATION )
				{
				        // save cal points

				    SAMPLE *p = pSamples ;
				    for ( int iSample=0 ; iSample < nSamples ; iSample++, p++ )
				    {
			    		fprintf ( outFile, "%+11.6f %+11.6f %+11.6f %4i %4i %3i %3i %3i %3i\n",
						    p->droX, p->droY, p->droZ, (int) p->h, (int) p->v,
					    	(int) p->gray, (int) p->red, (int) p->green, (int) p->blue ) ;
				    }
				}
				else if ( scanType == PART )
				{
                    SAMPLE *p = pSamples ;
				    for ( int iSample=0 ; iSample < nSamples ; iSample++, p++ )
				    {
                        double h = p->h ;
						double v = p->v ;
						double hh = h*h ;
						double hv = h*v ;
						double vv = v*v ;

							// calculate (x,y,z)

						double x = p->droX + A[0] + A[1]*h + A[2]*v + A[3]*hh + A[4]*hv + A[5]*vv ;
                        double y = p->droY + B[0] + B[1]*h + B[2]*v + B[3]*hh + B[4]*hv + B[5]*vv ;
                        double z = p->droZ + C[0] + C[1]*h + C[2]*v + C[3]*hh + C[4]*hv + C[5]*vv ;

						    // scale for user supplied scale factors

						x *= pScan3dDlg->xScale ;
						y *= pScan3dDlg->yScale ;
						z *= pScan3dDlg->zScale ;
						
						    // save (x,y,z,i) to file

						fprintf ( outFile, "%+11.6f %+11.6f %+11.6f ", x, y, z ) ;
						fprintf ( outFile, "%i ", (int) p->gray ) ;

//						fprintf ( outFile, "%+11.6f %+11.6f %+11.6f %4i %4i %3i %3i %3i %3i",
//						    p->droX, p->droY, p->droZ, (int) p->h, (int) p->v,
//					    	(int) p->gray, (int) p->red, (int) p->green, (int) p->blue ) ;
						fprintf ( outFile, "\n" ) ;
			        }
				}   
			}
	    }
    }

quit:
	if ( outFile )  //  close output file
	{
        fclose ( outFile ) ;
        outFile = NULL ;
	}
	StartTimer() ;  //  resume timed updates
	fRunning = FALSE ;  //  flag as finished
DbgMsg(("DoMotion() exit"));
    return(rtc) ;
}

int CalcCoef ( FILE *inFile )
{
	    // solve for coefficients for:
	    // Xcalc = Xm + A0 + A1*h + A2*v + A3*h*h + A4*h*v + A5*v*v
		// Ycalc = Ym + B0 + B1*h + B2*v + B3*h*h + B4*h*v + B5*v*v
		// Zcalc = Zm + C0 + C1*h + C2*v + C3*h*h + C4*h*v + C5*v*v

    int rtc = 0 ;

	    // create and initialize input data

    double Xp=0, Yp=0, Zp=0 ; // coordinates of pin point
    unsigned long n=0 ; // number of points
    double x=0, y=0, z=0 ; // dro values 
    int h=0, v=0 ; // CCD coordinates
    int gray=0, red=0, green=0, blue=0 ; // point intensities

	    // create and initialize sums

    double sumX=0, sumHX=0, sumVX=0, sumHHX=0, sumHVX=0, sumVVX=0 ;
    double sumY=0, sumHY=0, sumVY=0, sumHHY=0, sumHVY=0, sumVVY=0 ;
    double sumZ=0, sumHZ=0, sumVZ=0, sumHHZ=0, sumHVZ=0, sumVVZ=0 ;
    double sumH=0, sumV=0 ;
    double sumHH=0, sumHV=0, sumVV=0 ;
    double sumHHH=0, sumHHV=0, sumHVV=0, sumVVV=0 ;
    double sumHHHH=0, sumHHHV=0, sumHHVV=0, sumHVVV=0, sumVVVV=0 ;

	    // read the cal points and calculate sums

	while ( !feof(inFile) )
	{
	    fscanf ( inFile, "%lf %lf %lf %i %i %i %i %i %i\n",
	        &x, &y, &z, &h, &v, &gray, &red, &green, &blue ) ;

char tmp[255] ;
sprintf ( tmp, "%i %lf %lf %lf %i %i %i %i %i %i", n, x, y, z, h, v, gray, red, green, blue) ;
DbgMsg((tmp));

		sumX += x ;

		sumHX += h*x ;
        sumVX += v*x ;

		sumHHX += h*h*x ;
        sumHVX += h*v*x ;
        sumVVX += v*v*x ;

		sumY += y ;

		sumHY += h*y ;
        sumVY += v*y ;

		sumHHY += h*h*y ;
        sumHVY += h*v*y ;
        sumVVY += v*v*y ;

		sumZ += z ;

		sumHZ += h*z ;
        sumVZ += v*z ;

		sumHHZ += h*h*z ;
        sumHVZ += h*v*z ;
        sumVVZ += v*v*z ;

        sumH += h ;
        sumV += v ;

        sumHH += h*h ;
        sumHV += h*v ;
        sumVV += v*v ;

        sumHHH += h*h*h ;
        sumHHV += h*h*v ;
        sumHVV += h*v*v ;
        sumVVV += v*v*v ;

        sumHHHH += h*h*h*h ;
        sumHHHV += h*h*h*v ;
        sumHHVV += h*h*v*v ;
        sumHVVV += h*v*v*v ;
        sumVVVV += v*v*v*v ; 

		n++ ; // count the points
	}

    char tmp[81] ;
    sprintf ( tmp, "n=%lu", n ) ;
    DbgMsg((tmp)) ;

	    // create matrices for linear equations

    double M[nCoef][nCoef] ; // for calculating x, y, and z

	    // define matrices for linear equations

    A[0] = Xp*n - sumX ;
	A[1] = Xp*sumH - sumHX ;
	A[2] = Xp*sumV - sumVX ;
	A[3] = Xp*sumHH - sumHHX ;
    A[4] = Xp*sumHV - sumHVX ;
	A[5] = Xp*sumVV - sumVVX ;
			
	B[0] = Yp*n - sumY ;
	B[1] = Yp*sumH - sumHY ;
	B[2] = Yp*sumV - sumVY ;
	B[3] = Yp*sumHH - sumHHY ;
	B[4] = Yp*sumHV - sumHVY ;
	B[5] = Yp*sumVV - sumVVY ;

	C[0] = Zp*n - sumZ ;
	C[1] = Zp*sumH - sumHZ ;
	C[2] = Zp*sumV - sumVZ ;
	C[3] = Zp*sumHH - sumHHZ ;
	C[4] = Zp*sumHV - sumHVZ ;
	C[5] = Zp*sumVV - sumVVZ ;

    M[0][0] = n ;
    M[0][1] = sumH ;
    M[0][2] = sumV ;
    M[0][3] = sumHH ;
    M[0][4] = sumHV ;
    M[0][5] = sumVV ;   

    M[1][0] = sumH ;
    M[1][1] = sumHH ;
    M[1][2] = sumHV ;
    M[1][3] = sumHHH ;
    M[1][4] = sumHHV ;
    M[1][5] = sumHVV ;  

    M[2][0] = sumV ;
    M[2][1] = sumHV ;
    M[2][2] = sumVV ;
    M[2][3] = sumHHV ;
    M[2][4] = sumHVV ;
    M[2][5] = sumVVV ;      
  
    M[3][0] = sumHH ;
    M[3][1] = sumHHH ;
    M[3][2] = sumHHV ;
    M[3][3] = sumHHHH ;
    M[3][4] = sumHHHV ;
    M[3][5] = sumHHVV ;      

    M[4][0] = sumHV ;
    M[4][1] = sumHHV ;
    M[4][2] = sumHVV ;
    M[4][3] = sumHHHV ;
    M[4][4] = sumHHVV ;
    M[4][5] = sumHVVV ;      
  
    M[5][0] = sumVV ;
    M[5][1] = sumHVV ;
    M[5][2] = sumVVV ;
    M[5][3] = sumHHVV ;
    M[5][4] = sumHVVV ;
    M[5][5] = sumVVVV ; 

	    // show setup matrix

	int i, j, k ;  //  loop indeces
    DbgMsg(("Matrix i A[i] B[i] C[i] = M[i][j]"));
	for ( i=0 ; i<nCoef ; i++ )
    {
        char tmp[255] ;
        sprintf ( tmp, "%i %+14.6lg %+14.6g %+14.6g = %+14.6lg %+14.6lg %+14.6lg %+14.6lg %+14.6lg %+14.6lg",
            i, A[i], B[i], C[i], M[i][0], M[i][1], M[i][2], M[i][3], M[i][4], M[i][5] ) ;         
	    DbgMsg((tmp)) ;
	}

    for ( k=0 ; k<nCoef ; k++ )
	{
		    // make diagonal element equal one

		double div = M[k][k] ; 
        A[k] /= div ;	
		B[k] /= div ;
		C[k] /= div ;

		for ( j=0 ; j<nCoef ; j++ )
		    M[k][j] /= div ;

		    // make element below diagonal equal zero

	    for ( i=k+1 ; i<nCoef ; i++ )
		{
            double mult = M[i][k] ; 
            A[i] -= A[k] * mult ;
			B[i] -= B[k] * mult ;
			C[i] -= C[k] * mult ;

			for ( j=0 ; j<nCoef ; j++ )
                M[i][j] -= M[k][j] * mult ;
		} 
	}
	
	    // show matrix after first half done

    DbgMsg(("Matrix i A[i] B[i] C[i] = M[i][j]"));
	for ( i=0 ; i<nCoef ; i++ )
    {
        char tmp[255] ;
        sprintf ( tmp, "%i %+14.6lg %+14.6g %+14.6g = %+14.6lg %+14.6lg %+14.6lg %+14.6lg %+14.6lg %+14.6lg",
            i, A[i], B[i], C[i], M[i][0], M[i][1], M[i][2], M[i][3], M[i][4], M[i][5] ) ;         
	    DbgMsg((tmp)) ;
	}

        // make element above diagonal equal zero

	for ( k=nCoef-1 ; k>0 ; k-- )
	{
	    for ( i=k-1 ; i>=0 ; i-- )  
	    {
			double mult = M[i][k] ;
			A[i] -= A[k] * mult ;
			B[i] -= B[k] * mult ;
			C[i] -= C[k] * mult ;

			for ( j=0 ; j<nCoef ; j++ )
			{
                M[i][j] -= M[k][j] * mult ;
			} 
	    }
	}

		    // show matrix after second half done

    DbgMsg(("Matrix i A[i] B[i] C[i] = M[i][j]"));
	for ( i=0 ; i<nCoef ; i++ )
    {
        char tmp[255] ;
        sprintf ( tmp, "%i %+14.6lg %+14.6g %+14.6g = %+14.6lg %+14.6lg %+14.6lg %+14.6lg %+14.6lg %+14.6lg",
            i, A[i], B[i], C[i], M[i][0], M[i][1], M[i][2], M[i][3], M[i][4], M[i][5] ) ;         
	    DbgMsg((tmp)) ;
	}

quit:
	return(rtc) ;
}

int MakeTestCloud ( char *inFileName, char *outFileName )
{
    int rtc=0 ;
    FILE *inFile = NULL ;
	FILE *outFile = NULL ;
	int n=0 ;

    inFile = fopen ( inFileName, "r" ) ;
	if ( !inFile )
	{
        char tmp[255] ;
		sprintf ( tmp, "Cannot open [%s] for input", inFileName ) ;
		MessageBox ( NULL, tmp, NULL, MB_OK ) ;
		rtc = 1 ;
		goto quit ;
	}

    outFile = fopen ( outFileName, "w" ) ;
	if ( !outFile )
	{
        char tmp[255] ;
		sprintf ( tmp, "Cannot open [%s] for output", outFileName ) ;
		MessageBox ( NULL, tmp, NULL, MB_OK ) ;
		rtc = 2 ;
		goto quit ;
	}

	while ( !feof(inFile) )
	{
		double xDro, yDro, zDro ;
	    int h, v, gray, red, green, blue ;

	    fscanf ( inFile, "%lf %lf %lf %i %i %i %i %i %i\n",
	        &xDro, &yDro, &zDro, &h, &v, &gray, &red, &green, &blue ) ;

        char tmp[255] ;
        sprintf ( tmp, "%i %lf %lf %lf %i %i %i %i %i %i", n, xDro, yDro, zDro, h, v, gray, red, green, blue) ;
        DbgMsg((tmp));

		double x = xDro + A[0] + A[1]*h + A[2]*v + A[3]*h*h + A[4]*h*v + A[5]*v*v ;
        double y = yDro + B[0] + B[1]*h + B[2]*v + B[3]*h*h + B[4]*h*v + B[5]*v*v ;
		double z = zDro + C[0] + C[1]*h + C[2]*v + C[3]*h*h + C[4]*h*v + C[5]*v*v ;

		fprintf ( outFile, "%+11.6f %+11.6f %+11.6f", x, y, z ) ;

//		fprintf ( outFile, "%+11.6f %+11.6f %+11.6f %4i %4i %3i %3i %3i %3i",
//		    p->droX, p->droY, p->droZ, (int) p->h, (int) p->v,
//		   	(int) p->gray, (int) p->red, (int) p->green, (int) p->blue ) ;
		fprintf ( outFile, "\n" ) ;

		n++ ;
	}

quit:
	if ( inFile )
	{
		fclose ( inFile ) ;
		inFile = NULL ;
	}

	if ( outFile )
	{
		fclose ( outFile ) ;
		outFile = NULL ;
	}

	return(rtc) ;
}

void CalcFov ( void ) 
{
	DbgMsg(("CalcFov entry"));

	double xDro=0, yDro=0, zDro=0 ;
    double hSize = bmiImage1.biWidth ;
	double vSize = bmiImage1.biHeight ;

	    // Calculate horizontal field of view

	double v = vSize/2 ;

	double h = hSize ;
    double x1 = xDro + A[0] + A[1]*h + A[2]*v + A[3]*h*h + A[4]*h*v + A[5]*v*v ;
	double y1 = yDro + B[0] + B[1]*h + B[2]*v + B[3]*h*h + B[4]*h*v + B[5]*v*v ;
	double z1 = zDro + C[0] + C[1]*h + C[2]*v + C[3]*h*h + C[4]*h*v + C[5]*v*v ;

	h = 0 ;
	double x2 = xDro + A[0] + A[1]*h + A[2]*v + A[3]*h*h + A[4]*h*v + A[5]*v*v ;
	double y2 = yDro + B[0] + B[1]*h + B[2]*v + B[3]*h*h + B[4]*h*v + B[5]*v*v ;
	double z2 = zDro + C[0] + C[1]*h + C[2]*v + C[3]*h*h + C[4]*h*v + C[5]*v*v ;

	double dx12 = x2 - x1 ;
	double dy12 = y2 - y1 ;
	double dz12 = z2 - z1 ;

	pScan3dDlg->hfov = sqrt( dx12*dx12 + dy12*dy12 + dz12*dz12 ) ;

	    // Calculate vertical vield of view

	h = hSize/2 ;

	v = vSize ;
	double x3 = xDro + A[0] + A[1]*h + A[2]*v + A[3]*h*h + A[4]*h*v + A[5]*v*v ;
	double y3 = yDro + B[0] + B[1]*h + B[2]*v + B[3]*h*h + B[4]*h*v + B[5]*v*v ;
	double z3 = zDro + C[0] + C[1]*h + C[2]*v + C[3]*h*h + C[4]*h*v + C[5]*v*v ;

	v = 0 ;
	double x4 = xDro + A[0] + A[1]*h + A[2]*v + A[3]*h*h + A[4]*h*v + A[5]*v*v ;
	double y4 = yDro + B[0] + B[1]*h + B[2]*v + B[3]*h*h + B[4]*h*v + B[5]*v*v ;
	double z4 = zDro + C[0] + C[1]*h + C[2]*v + C[3]*h*h + C[4]*h*v + C[5]*v*v ;

	double dx34 = x4 - x3 ;
	double dy34 = y4 - y3 ;
	double dz34 = z4 - z3 ;
	
	pScan3dDlg->vfov = sqrt( dx34*dx34 + dy34*dy34 + dz34*dz34 ) ;

	    // Calculate azimuth and elevation

	double rxy = sqrt( dx34*dx34 + dy34*dy34 ) ;
	pScan3dDlg->elevation = atan2( dz34, rxy ) * degPerRad ;
	pScan3dDlg->azimuth = atan2 ( dy34, dx34 ) * degPerRad ;

	DbgMsg(("CalcFov exit"));
    return ;
}


// end video.cpp